"use client";
import { useState, useEffect, useRef } from "react";
import { usePathname } from "next/navigation";
import Link from "next/link";
import styles from "./header.module.scss";
import { Logo, LogoIcon } from "@/components/general/Logo";

export const Header = () => {
  const [showMenu, setShowMenu] = useState(false);
  function toggleMenu() {
    setShowMenu(!showMenu);
  }
  const siteMenu = useRef<HTMLInputElement>(null);
  useEffect(() => {
    // only add the event listener when the siteMenu is opened
    if (!showMenu) return;
    function handleClick(event: Event) {
      if (
        siteMenu.current &&
        !siteMenu.current.contains(event.target as Node)
      ) {
        setShowMenu(false);
        setActiveMenuItem(undefined);
      }
    }
    window.addEventListener("click", handleClick);
    // clean up
    return () => window.removeEventListener("click", handleClick);
  }, [showMenu]);

  useEffect(() => {
    if (showMenu) {
      document.body.style.overflow = "hidden";
      document.body.style.height = "100vh";
    } else {
      document.body.style.overflow = "";
      document.body.style.height = "";
    }
  }, [showMenu]);

  const pathname = usePathname();
  useEffect(() => {
    // hide menu on path change
    setShowMenu(false);
    setActiveMenuItem(undefined);
  }, [pathname]);

  const [activeMenuItem, setActiveMenuItem] = useState<string | undefined>(
    undefined
  );

  return (
    <header className={styles.header}>
      <Link href="/" aria-label="Hjem">
        <Logo />
      </Link>
      <nav className={styles.siteMenu} data-show={showMenu} ref={siteMenu}>
        <ul className={styles.mainMenu}>
          <li>
            <Link
              href="/arrangementer"
              data-active={pathname === "/arrangementer"}
            >
              Arrangementer
            </Link>
          </li>
          <li>
            <Link href="/praktisk" data-active={pathname === "/praktisk"}>
              Praktisk info
            </Link>
          </li>
          <li>
            <Link href="/utleie" data-active={pathname === "/utleie"}>
              Utleie
            </Link>
          </li>
          <li>
            <Link href="/medlem" data-active={pathname === "/medlem"}>
              Bli medlem
            </Link>
          </li>
        </ul>
        <button
          className={styles.toggleMenu}
          aria-label="Vis meny"
          onClick={toggleMenu}
        >
          <MenuIcon showMenu={showMenu} />
        </button>
        <div className={styles.fullMenu}>
          <LogoIcon face="right" />
          <ul>
            <li className={styles.menuItemLarge}>
              <Link
                href="/arrangementer"
                data-active={pathname === "/arrangementer"}
              >
                Arrangementer
              </Link>
            </li>
            <li className={styles.menuItemLarge}>
              <Link href="/praktisk" data-active={pathname === "/praktisk"}>
                Praktisk info
              </Link>
            </li>
            <li className={styles.menuItemLarge}>
              <Link href="/utleie" data-active={pathname === "/utleie"}>
                Utleie
              </Link>
            </li>
            <li className={styles.menuItemLarge}>
              <Link href="/medlem" data-active={pathname === "/medlem"}>
                Bli medlem
              </Link>
            </li>
            <li className={styles.divider}>
              <Link href="/aktuelt" data-active={pathname === "/aktuelt"}>
                Siste nytt
              </Link>
            </li>
            <li>
              <Link href="/om" data-active={pathname === "/om"}>
                Om oss
              </Link>
            </li>
            <li>
              <Link href="/kontakt" data-active={pathname === "/kontakt"}>
                Kontakt
              </Link>
            </li>
            <li>
              <Link href="/" data-active={pathname === "/frivillig"}>
                Bli frivillig
              </Link>
            </li>
            <li className={styles.search}>
              <label>
                <p>Søk</p>
                <input type="text" />
              </label>
            </li>
            <li className={styles.galtinn}>
              <a href="https://galtinn.neuf.no/" target="_blank">
                Mitt medlemskap &rarr;
              </a>
            </li>
            <li>
              {/* TODO: skal trolig ikke være så synlig ved lansering */}
              <Link href="/lokaler">Lokaler*</Link>
            </li>
            <li>
              {/* TODO: skal trolig ikke være så synlig ved lansering */}
              <Link href="/foreninger">Foreninger*</Link>
            </li>
          </ul>
        </div>
      </nav>
    </header>
  );
};

export const MenuIcon = ({ showMenu }: { showMenu: boolean }) => {
  return (
    <div className={styles.menuIcon}>
      {showMenu ? (
        <svg
          viewBox="0 0 309 309"
          fill="none"
          xmlns="http://www.w3.org/2000/svg"
        >
          <path
            d="M154.2 308.4C239.362 308.4 308.4 239.362 308.4 154.2C308.4 69.0377 239.362 0 154.2 0C69.0377 0 0 69.0377 0 154.2C0 239.362 69.0377 308.4 154.2 308.4Z"
            fill="var(--color-menu-text)"
          />
          <rect
            x="101.964"
            y="80.8135"
            width="177.657"
            height="29.9116"
            transform="rotate(45 101.964 80.8135)"
            fill="var(--color-menu-background)"
          />
          <rect
            x="227.586"
            y="101.964"
            width="177.657"
            height="29.9116"
            transform="rotate(135 227.586 101.964)"
            fill="var(--color-menu-background)"
          />
        </svg>
      ) : (
        <svg
          viewBox="0 0 309 309"
          fill="none"
          xmlns="http://www.w3.org/2000/svg"
        >
          <path
            d="M154.2 308.4C239.362 308.4 308.4 239.362 308.4 154.2C308.4 69.0377 239.362 0 154.2 0C69.0377 0 0 69.0377 0 154.2C0 239.362 69.0377 308.4 154.2 308.4Z"
            fill="var(--color-text)"
          />
          <rect
            x="65.3716"
            y="139.244"
            width="177.657"
            height="29.9116"
            fill="var(--color-background)"
          />
          <rect
            x="65.3716"
            y="199.244"
            width="177.657"
            height="29.9116"
            fill="var(--color-background)"
          />
          <rect
            x="65.3716"
            y="79.2444"
            width="177.657"
            height="29.9116"
            fill="var(--color-background)"
          />
        </svg>
      )}
    </div>
  );
};
